/**  Struct for handling an array of VOID Pointers.
     The main idea was, that you convert (cast) <b>any</b> type
     of pointers to a VOIDP - so you can use this array for any
     pointers to structs.

@author Hans-Peter Jacobs
@version $Id: hpj_parray.h,v 1.2 2000/01/17 23:59:28 jacobs Exp $

@name hpj_parray
*/
//@{
#ifndef __HPJ_PARRAY_H__
#define __HPJ_PARRAY_H__

#ifndef INT32
#define INT32 int
#endif

#ifndef VOIDP
#define VOIDP void*
#endif

#ifndef def_boolean
#define def_boolean
typedef enum {false=0, true=1} boolean;
#endif

/** This Macro should be used for casting <b>to</b> VOIDP.
    The cast back to the original Pointer has to be done
    seperately.*/
#ifndef P2VOIDP
#define P2VOIDP(a) ((VOIDP) a)
#endif

/** @name hpj_parray - name of the struct for external use*/
//@{
typedef struct
{
  VOIDP  *mem;
  INT32   length;
  INT32   capacity;
}hpj_parray; 
//@}











/** @name Constructors and destructors*/
//@{
/** Creates a new struct. There will be space for "initial_size" elemets.
    If this bound is reached "initial_size" * 2 bytes will be allocated.. etc.
    @param initial_size Sets the initial memory allocated for elems.
    @return pointer to the new created struct*/
hpj_parray *parray_new(INT32 initial_size);


/** Creates a new hpj_parray from an existing.
    @param source existing hpj_parray
    @return newly created hpj_parray, copied from the source
*/
hpj_parray *parray_clone(hpj_parray *source);

/** Releases all memory allocated for the hpj_parray.
    It returns NULL, so the calling function can assign
    the result to the old "hpj_parray"-pointer.
    @param hpj_iarray pointer to the parray-structure
    @return NULL
*/
hpj_parray *parray_del(hpj_parray *parray);
//@}


/** @name Functions */
//@{
/** Sets the size of the array to the new size. The region between the
    start and the end of the array will be "touched", can be accessed
    via <b>parray_safe_set</b>.
    @param parray pointer to parray to be used for operation
    @param elems number of elems space is needed for
    @return true for success, false for memory-failure*/
boolean parray_expand(hpj_parray *parray, INT32 elems);

/** Returns the number of elements stored within the parray.
    This indicates just the last "touched" position within the array.
    @param parray pointer to the desired hpj_parray-struct
    @return number of elements within the parray*/
INT32 parray_elements(hpj_parray *parray);

/** Returns the length of the iarray-memory-block.
    Please note - this differs from the number of elements stored !!!
    @param iarray pointer to the hpj_iarray-struct
    @return memory used by the struct and the allocated memory-block*/
INT32 parray_size(hpj_parray *parray);

/** Returns the element at the given position.
    @param parray pointer to the struct
    @param pos position of the desired element
    @return value at the position*/
VOIDP parray_get(hpj_parray *parray, INT32 pos);

/** Sets the given pointer at the position within the parray.
    <b>WARNING</b> : since the memory will be allocated if not enough
    you can rip "holes" into your parray.<br>
    A call for the length returns the last know position, holes between
    the elements are handled in no special manner.
    @param parray pointer to the struct
    @param pointer pointer to be set
    @param pos position of the desired element
    @return true for success, false else*/
boolean parray_set(hpj_parray *parray, VOIDP pointer, INT32 pos);

/** Sets the given pointer at the position within the parray.
    If the pos is out of the bounds (never before a value was set
    in this region) the value will not be set -> an error will be
    displayed on <b>stderr</b> and <b>false</b> will be returned
    @param parray pointer to the struct
    @param pointer pointer to be set
    @param pos position of the desired element
    @return true for success, false else*/
boolean parray_safe_set(hpj_parray *parray, VOIDP pointer, INT32 pos);

/** This adds a new VOIDP to the existing memory-block.
    
  @param parray existing iarray
  @param value new VOIDP value
  @return boolean value (true/false) for success*/
boolean parray_append(hpj_parray *parray, VOIDP pointer);

/** Displays important infos of the parray on stdout.
    @param parray hpj_parray for output
    @return true or false for success*/
boolean parray_show(hpj_parray *parray);
//@}

#endif
//@}
